@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 #include <assert.h>
 #include <errno.h>
@@ -13,19 +14,24 @@
 #include <libudev.h>
 
 #define LOG_MODULE "battery"
-#define LOG_ENABLE_DBG 0
+#define LOG_ENABLE_DBG 1
 #include "../log.h"
 #include "../bar/bar.h"
 #include "../config.h"
 #include "../config-verify.h"
 #include "../plugin.h"
 
+#define max(x, y) ((x) > (y) ? (x) : (y))
+
+static const long min_poll_interval = 250;
+static const long default_poll_interval = 60 * 1000;
+
 enum state { STATE_FULL, STATE_NOTCHARGING, STATE_CHARGING, STATE_DISCHARGING, STATE_UNKNOWN };
 
 struct private {
     struct particle *label;
 
-    int poll_interval;
+    long poll_interval;
     char *battery;
     char *manufacturer;
     char *model;
@@ -41,8 +47,25 @@ struct private {
     long charge;
     long current;
     long time_to_empty;
+    long time_to_full;
 };
 
+static void
+timespec_sub(const struct timespec *a, const struct timespec *b,
+             struct timespec *res)
+{
+    const long one_sec_in_ns = 1000000000;
+
+    res->tv_sec = a->tv_sec - b->tv_sec;
+    res->tv_nsec = a->tv_nsec - b->tv_nsec;
+
+    /* tv_nsec may be negative */
+    if (res->tv_nsec < 0) {
+        res->tv_sec--;
+        res->tv_nsec += one_sec_in_ns;
+    }
+}
+
 static void
 destroy(struct module *mod)
 {
@@ -82,9 +105,14 @@ content(struct module *mod)
     unsigned long hours;
     unsigned long minutes;
 
-    if (m->time_to_empty >= 0) {
-        hours = m->time_to_empty / 60;
-        minutes = m->time_to_empty % 60;
+    if (m->time_to_empty > 0) {
+        minutes = m->time_to_empty / 60;
+        hours = minutes / 60;
+        minutes = minutes % 60;
+    } else if (m->time_to_full > 0) {
+        minutes = m->time_to_full / 60;
+        hours = minutes / 60;
+        minutes = minutes % 60;
     } else if  (m->energy_full >= 0 && m->charge && m->power >= 0) {
         unsigned long energy = m->state == STATE_CHARGING
             ? m->energy_full - m->energy : m->energy;
@@ -147,20 +175,18 @@ content(struct module *mod)
 }
 
 static const char *
-readline_from_fd(int fd)
+readline_from_fd(int fd, size_t sz, char buf[static sz])
 {
-    static char buf[4096];
-
-    ssize_t sz = read(fd, buf, sizeof(buf) - 1);
+    ssize_t bytes = read(fd, buf, sz - 1);
     lseek(fd, 0, SEEK_SET);
 
-    if (sz < 0) {
+    if (bytes < 0) {
         LOG_WARN("failed to read from FD=%d", fd);
         return NULL;
     }
 
-    buf[sz] = '\0';
-    for (ssize_t i = sz - 1; i >= 0 && buf[i] == '\n'; sz--)
+    buf[bytes] = '\0';
+    for (ssize_t i = bytes - 1; i >= 0 && buf[i] == '\n'; bytes--)
         buf[i] = '\0';
 
     return buf;
@@ -169,7 +195,8 @@ readline_from_fd(int fd)
 static long
 readint_from_fd(int fd)
 {
-    const char *s = readline_from_fd(fd);
+    char buf[512];
+    const char *s = readline_from_fd(fd, sizeof(buf), buf);
     if (s == NULL)
         return 0;
 
@@ -186,6 +213,8 @@ readint_from_fd(int fd)
 static bool
 initialize(struct private *m)
 {
+    char line_buf[512];
+
     int pw_fd = open("/sys/class/power_supply", O_RDONLY);
     if (pw_fd < 0) {
         LOG_ERRNO("/sys/class/power_supply");
@@ -207,7 +236,7 @@ initialize(struct private *m)
                      m->battery, strerror(errno));
             m->manufacturer = NULL;
         } else {
-            m->manufacturer = strdup(readline_from_fd(fd));
+            m->manufacturer = strdup(readline_from_fd(fd, sizeof(line_buf), line_buf));
             close(fd);
         }
     }
@@ -219,7 +248,7 @@ initialize(struct private *m)
                      m->battery, strerror(errno));
             m->model = NULL;
         } else {
-            m->model = strdup(readline_from_fd(fd));
+            m->model = strdup(readline_from_fd(fd, sizeof(line_buf), line_buf));
             close(fd);
         }
     }
@@ -327,6 +356,7 @@ update_status(struct module *mod)
     int charge_fd = openat(base_dir_fd, "charge_now", O_RDONLY);
     int current_fd = openat(base_dir_fd, "current_now", O_RDONLY);
     int time_to_empty_fd = openat(base_dir_fd, "time_to_empty_now", O_RDONLY);
+    int time_to_full_fd = openat(base_dir_fd, "time_to_full_now", O_RDONLY);
 
     long capacity = readint_from_fd(capacity_fd);
     long energy = energy_fd >= 0 ? readint_from_fd(energy_fd) : -1;
@@ -334,8 +364,10 @@ update_status(struct module *mod)
     long charge = charge_fd >= 0 ? readint_from_fd(charge_fd) : -1;
     long current = current_fd >= 0 ? readint_from_fd(current_fd) : -1;
     long time_to_empty = time_to_empty_fd >= 0 ? readint_from_fd(time_to_empty_fd) : -1;
+    long time_to_full = time_to_full_fd >= 0 ? readint_from_fd(time_to_full_fd) : -1;
 
-    const char *status = readline_from_fd(status_fd);
+    char buf[512];
+    const char *status = readline_from_fd(status_fd, sizeof(buf), buf);
 
     if (status_fd >= 0)
         close(status_fd);
@@ -351,6 +383,8 @@ update_status(struct module *mod)
         close(current_fd);
     if (time_to_empty_fd >= 0)
         close(time_to_empty_fd);
+    if (time_to_full_fd >= 0)
+        close(time_to_full_fd);
     if (base_dir_fd >= 0)
         close(base_dir_fd);
 
@@ -375,8 +409,8 @@ update_status(struct module *mod)
     }
 
     LOG_DBG("capacity: %ld, energy: %ld, power: %ld, charge=%ld, current=%ld, "
-            "time-to-empty: %ld", capacity, energy, power, charge, current,
-            time_to_empty);
+            "time-to-empty: %ld, time-to-full: %ld", capacity, energy, power,
+            charge, current, time_to_empty, time_to_full);
 
     mtx_lock(&mod->lock);
     m->state = state;
@@ -386,6 +420,7 @@ update_status(struct module *mod)
     m->charge = charge;
     m->current = current;
     m->time_to_empty = time_to_empty;
+    m->time_to_full = time_to_full;
     mtx_unlock(&mod->lock);
     return true;
 }
@@ -423,14 +458,25 @@ run(struct module *mod)
 
     bar->refresh(bar);
 
+    int timeout_left_ms = m->poll_interval;
+
     while (true) {
         struct pollfd fds[] = {
             {.fd = mod->abort_fd, .events = POLLIN},
             {.fd = udev_monitor_get_fd(mon), .events = POLLIN},
         };
-        if (poll(fds, sizeof(fds) / sizeof(fds[0]),
-                 m->poll_interval > 0 ? m->poll_interval * 1000 : -1) < 0)
-        {
+
+        int timeout = m->poll_interval > 0 ? timeout_left_ms : -1;
+
+        struct timespec time_before_poll;
+        if (clock_gettime(CLOCK_BOOTTIME, &time_before_poll) < 0) {
+            LOG_ERRNO("failed to get current time");
+            break;
+        }
+
+        const int poll_ret = poll(fds, sizeof(fds) / sizeof(fds[0]), timeout);
+
+        if (poll_ret < 0) {
             if (errno == EINTR)
                 continue;
 
@@ -443,21 +489,54 @@ run(struct module *mod)
             break;
         }
 
+        bool udev_for_us = false;
+
         if (fds[1].revents & POLLIN) {
             struct udev_device *dev = udev_monitor_receive_device(mon);
-            if (dev == NULL)
-                continue;
+            if (dev != NULL) {
+                const char *sysname = udev_device_get_sysname(dev);
+                udev_for_us =
+                    sysname != NULL && strcmp(sysname, m->battery) == 0;
 
-            const char *sysname = udev_device_get_sysname(dev);
-            bool is_us = sysname != NULL && strcmp(sysname, m->battery) == 0;
-            udev_device_unref(dev);
+                if (!udev_for_us) {
+                    LOG_DBG("udev notification not for us (%s != %s)",
+                            m->battery, sysname != sysname ? sysname : "NULL");
+                } else
+                    LOG_DBG("triggering update due to udev notification");
 
-            if (!is_us)
-                continue;
+                udev_device_unref(dev);
+            }
         }
 
-        if (update_status(mod))
-            bar->refresh(bar);
+        if (udev_for_us || poll_ret == 0) {
+            if (update_status(mod))
+                bar->refresh(bar);
+        }
+
+        if (poll_ret == 0 || udev_for_us) {
+            LOG_DBG("resetting timeout-left to %ldms", m->poll_interval);
+            timeout_left_ms = m->poll_interval;
+        } else {
+            struct timespec time_after_poll;
+            if (clock_gettime(CLOCK_BOOTTIME, &time_after_poll) < 0) {
+                LOG_ERRNO("failed to get current time");
+                break;
+            }
+
+            struct timespec timeout_consumed;
+            timespec_sub(&time_after_poll, &time_before_poll, &timeout_consumed);
+
+            const int timeout_consumed_ms =
+                timeout_consumed.tv_sec * 1000 + timeout_consumed.tv_nsec / 1000000;
+
+            LOG_DBG("timeout-left before: %dms, consumed: %dms, updated: %dms",
+                    timeout_left_ms, timeout_consumed_ms,
+                    max(timeout_left_ms - timeout_consumed_ms, 0));
+
+            timeout_left_ms -= timeout_consumed_ms;
+            if (timeout_left_ms < 0)
+                timeout_left_ms = 0;
+        }
     }
 
 out:
@@ -469,11 +548,11 @@ out:
 }
 
 static struct module *
-battery_new(const char *battery, struct particle *label, int poll_interval_secs)
+battery_new(const char *battery, struct particle *label, long poll_interval_msecs)
 {
     struct private *m = calloc(1, sizeof(*m));
     m->label = label;
-    m->poll_interval = poll_interval_secs;
+    m->poll_interval = poll_interval_msecs;
     m->battery = strdup(battery);
     m->state = STATE_UNKNOWN;
 
@@ -496,7 +575,26 @@ from_conf(const struct yml_node *node, struct conf_inherit inherited)
     return battery_new(
         yml_value_as_string(name),
         conf_to_particle(c, inherited),
-        poll_interval != NULL ? yml_value_as_int(poll_interval) : 60);
+        (poll_interval != NULL
+         ? yml_value_as_int(poll_interval)
+         : default_poll_interval));
+}
+
+static bool
+conf_verify_poll_interval(keychain_t *chain, const struct yml_node *node)
+{
+    if (!conf_verify_unsigned(chain, node))
+        return false;
+
+    const long value = yml_value_as_int(node);
+
+    if (value != 0 && value < min_poll_interval) {
+        LOG_ERR("%s: interval value cannot be less than %ldms",
+                conf_err_prefix(chain, node), min_poll_interval);
+        return false;
+    }
+
+    return true;
 }
 
 static bool
@@ -504,7 +602,7 @@ verify_conf(keychain_t *chain, const struct yml_node *node)
 {
     static const struct attr_info attrs[] = {
         {"name", true, &conf_verify_string},
-        {"poll-interval", false, &conf_verify_unsigned},
+        {"poll-interval", false, &conf_verify_poll_interval},
         MODULE_COMMON_ATTRS,
     };
 
