@@ -12,13 +12,14 @@
 
 #define LOG_MODULE "mem"
 #define LOG_ENABLE_DBG 0
-#define SMALLEST_INTERVAL 500
 #include "../bar/bar.h"
 #include "../config-verify.h"
 #include "../config.h"
 #include "../log.h"
 #include "../plugin.h"
 
+static const long min_poll_interval = 250;
+
 struct private
 {
     struct particle *label;
@@ -151,20 +152,23 @@ mem_new(uint16_t interval, struct particle *label)
 static struct module *
 from_conf(const struct yml_node *node, struct conf_inherit inherited)
 {
-    const struct yml_node *interval = yml_get_value(node, "interval");
+    const struct yml_node *interval = yml_get_value(node, "poll-interval");
     const struct yml_node *c = yml_get_value(node, "content");
 
-    return mem_new(interval == NULL ? SMALLEST_INTERVAL : yml_value_as_int(interval), conf_to_particle(c, inherited));
+    return mem_new(
+        interval == NULL ? min_poll_interval : yml_value_as_int(interval),
+        conf_to_particle(c, inherited));
 }
 
 static bool
-conf_verify_interval(keychain_t *chain, const struct yml_node *node)
+conf_verify_poll_interval(keychain_t *chain, const struct yml_node *node)
 {
     if (!conf_verify_unsigned(chain, node))
         return false;
 
-    if (yml_value_as_int(node) < SMALLEST_INTERVAL) {
-        LOG_ERR("%s: interval value cannot be less than %d ms", conf_err_prefix(chain, node), SMALLEST_INTERVAL);
+    if (yml_value_as_int(node) < min_poll_interval) {
+        LOG_ERR("%s: interval value cannot be less than %ldms",
+                conf_err_prefix(chain, node), min_poll_interval);
         return false;
     }
 
@@ -175,7 +179,7 @@ static bool
 verify_conf(keychain_t *chain, const struct yml_node *node)
 {
     static const struct attr_info attrs[] = {
-        {"interval", false, &conf_verify_interval},
+        {"poll-interval", false, &conf_verify_poll_interval},
         MODULE_COMMON_ATTRS,
     };
 
