@@ -42,6 +42,7 @@ struct private {
     char *album;
     char *artist;
     char *title;
+    char *file;
 
     struct {
         uint64_t value;
@@ -75,6 +76,7 @@ destroy(struct module *mod)
     free(m->album);
     free(m->artist);
     free(m->title);
+    free(m->file);
     assert(m->conn == NULL);
 
     m->label->destroy(m->label);
@@ -84,7 +86,7 @@ destroy(struct module *mod)
 }
 
 static const char *
-description(struct module *mod)
+description(const struct module *mod)
 {
     return "mpd";
 }
@@ -173,13 +175,14 @@ content(struct module *mod)
             tag_new_string(mod, "album", m->album),
             tag_new_string(mod, "artist", m->artist),
             tag_new_string(mod, "title", m->title),
+            tag_new_string(mod, "file", m->file),
             tag_new_string(mod, "pos", pos),
             tag_new_string(mod, "end", end),
             tag_new_int(mod, "duration", m->duration),
             tag_new_int_realtime(
                 mod, "elapsed", elapsed, 0, m->duration, realtime),
         },
-        .count = 12,
+        .count = 13,
     };
 
     mtx_unlock(&mod->lock);
@@ -223,7 +226,7 @@ wait_for_socket_create(const struct module *mod)
     struct stat st;
     if (stat(m->host, &st) == 0 && S_ISSOCK(st.st_mode)) {
 
-        int s = socket(AF_UNIX, SOCK_STREAM, 0);
+        int s = socket(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0);
 
         struct sockaddr_un addr = {.sun_family = AF_UNIX};
         strncpy(addr.sun_path, m->host, sizeof(addr.sun_path) - 1);
@@ -354,20 +357,24 @@ update_status(struct module *mod)
         free(m->album); m->album = NULL;
         free(m->artist); m->artist = NULL;
         free(m->title); m->title = NULL;
+        free(m->file); m->file = NULL;
         mtx_unlock(&mod->lock);
     } else {
         const char *album = mpd_song_get_tag(song, MPD_TAG_ALBUM, 0);
         const char *artist = mpd_song_get_tag(song, MPD_TAG_ARTIST, 0);
         const char *title = mpd_song_get_tag(song, MPD_TAG_TITLE, 0);
+        const char *file = mpd_song_get_uri(song);
 
         mtx_lock(&mod->lock);
         free(m->album);
         free(m->artist);
         free(m->title);
+        free(m->file);
 
         m->album = album != NULL ? strdup(album) : NULL;
         m->artist = artist != NULL ? strdup(artist) : NULL;
         m->title = title != NULL ? strdup(title) : NULL;
+        m->file = file != NULL ? strdup(file) : NULL;
         mtx_unlock(&mod->lock);
 
         mpd_song_free(song);
@@ -397,6 +404,7 @@ run(struct module *mod)
         free(m->album); m->album = NULL;
         free(m->artist); m->artist = NULL;
         free(m->title); m->title = NULL;
+        free(m->file); m->file = NULL;
         m->state = MPD_STATE_UNKNOWN;
         m->elapsed.value = m->duration = 0;
         m->elapsed.when.tv_sec = m->elapsed.when.tv_nsec = 0;
